# Copyright (C) 2021  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from typing import Any, Dict, Iterable

from swh.counters.redis import Redis


def process_journal_messages(
    messages: Dict[str, Iterable[Any]], *, counters: Redis
) -> None:
    """Worker function for `JournalClient.process(worker_fn)`"""

    for key in messages.keys():
        counters.add(key, messages[key])



# Copyright (C) 2021  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import pytest
from redis import Redis as RedisClient

from swh.counters.redis import DEFAULT_REDIS_PORT, Redis


def test__redis__constructor():
    r = Redis("fakehost")
    assert r.host == "fakehost"
    assert r.port == DEFAULT_REDIS_PORT

    r = Redis("host:11")
    assert r.host == "host"
    assert r.port == 11

    with pytest.raises(ValueError, match="url"):
        Redis("fake:host:port")


def test__redis__only_one_client_instantiation(mocker):
    mock = mocker.patch("swh.counters.redis.RedisClient")

    r = Redis("redishost:1234")

    # ensure lazy loading
    assert r._redis_client is None

    client = r.redis_client

    assert mock.call_count == 1
    args = mock.call_args[1]
    assert args["host"] == "redishost"
    assert args["port"] == 1234
    assert r._redis_client is not None

    client2 = r.redis_client
    assert mock.call_count == 1
    assert client == client2


def test__redis__ping_ko():
    r = Redis("wronghost")
    assert r.check() is False


def test__redis__ping_ok(local_redis):
    r = Redis("%s:%d" % (local_redis.host, local_redis.port))
    assert r.check() is True


def test__redis__collection(local_redis):
    r = Redis("%s:%d" % (local_redis.host, local_redis.port))
    r.add("c1", [b"k1", b"k2", b"k3"])
    r.add("c2", [b"k1"])
    r.add("c3", [b"k2"])
    r.add("c3", [b"k5"])

    assert 3 == r.get_count("c1")
    assert 1 == r.get_count("c2")
    assert 2 == r.get_count("c3")
    assert 0 == r.get_count("c4")


def test__redis__collections(local_redis):
    client = RedisClient(host=local_redis.host, port=local_redis.port)
    client.pfadd("counter1", b"k1")
    client.pfadd("counter2", b"k2")

    r = Redis("%s:%d" % (local_redis.host, local_redis.port))

    counters = r.get_counters()

    assert 2 == len(counters)
    assert b"counter1" in counters
    assert b"counter2" in counters



# Copyright (C) 2021  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

from swh.core.api import RPCClient

from ..interface import CountersInterface


class RemoteCounters(RPCClient):
    """Proxy to a remote counters API"""

    backend_class = CountersInterface



# Copyright (C) 2021  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import click

from swh.core.cli import CONTEXT_SETTINGS
from swh.core.cli import swh as swh_cli_group
from swh.counters.kafka_client import KeyOrientedJournalClient


@swh_cli_group.group(name="counters", context_settings=CONTEXT_SETTINGS)
@click.option(
    "--config-file",
    "-C",
    default=None,
    type=click.Path(exists=True, dir_okay=False,),
    help="Configuration file.",
)
@click.pass_context
def counters_cli_group(ctx, config_file):
    """Software Heritage Counters tools."""
    from swh.core import config

    ctx.ensure_object(dict)
    conf = config.read(config_file)
    ctx.obj["config"] = conf


@counters_cli_group.command("journal-client")
@click.option(
    "--stop-after-objects",
    "-m",
    default=None,
    type=int,
    help="Maximum number of objects to replay. Default is to run forever.",
)
@click.option(
    "--object-type",
    "-o",
    multiple=True,
    help="Default list of object types to subscribe to",
)
@click.option(
    "--prefix", "-p", help="Topic prefix to use (e.g swh.journal.objects)",
)
@click.pass_context
def journal_client(ctx, stop_after_objects, object_type, prefix):
    """Listens for new messages from the SWH Journal, and count them
    `"""
    import functools

    from . import get_counters
    from .journal_client import process_journal_messages

    config = ctx.obj["config"]
    journal_cfg = config["journal"]

    journal_cfg["object_types"] = object_type or journal_cfg.get("object_types", [])
    journal_cfg["prefix"] = prefix or journal_cfg.get("prefix")
    journal_cfg["stop_after_objects"] = stop_after_objects or journal_cfg.get(
        "stop_after_objects"
    )

    if len(journal_cfg["object_types"]) == 0:
        raise ValueError("'object_types' must be specified by cli or configuration")

    if journal_cfg["prefix"] is None:
        raise ValueError("'prefix' must be specified by cli or configuration")

    client = KeyOrientedJournalClient(**journal_cfg,)

    counters = get_counters(**config["counters"])

    worker_fn = functools.partial(process_journal_messages, counters=counters,)
    nb_messages = 0
    try:
        nb_messages = client.process(worker_fn)
        print("Processed %d messages." % nb_messages)
    except KeyboardInterrupt:
        ctx.exit(0)
    else:
        print("Done.")
    finally:
        client.close()
